@@ -1,5 +1,4 @@
-use tract_core::internal::*;
-use tract_core::ops::array::Slice;
+use tract_nnef::internal::*;
 
 #[derive(Debug, Clone, Default, Hash)]
 pub struct PulsedAxisSlice {
@@ -23,9 +22,20 @@ impl Op for PulsedAxisSlice {
     not_a_typed_op!();
 }
 
+impl TypedOp for PulsedAxisSlice {
+    fn output_facts(&self, inputs: &[&TypedFact]) -> TractResult<TVec<TypedFact>> {
+        Ok(tvec!(inputs[0].clone()))
+    }
+
+    as_op!();
+}
+
 impl EvalOp for PulsedAxisSlice {
+    fn is_stateless(&self) -> bool {
+        false
+    }
+
     fn eval(&self, inputs: TVec<Arc<Tensor>>) -> TractResult<TVec<Arc<Tensor>>> {
         Ok(inputs)
     }
 }
-
